@@ -4,7 +4,7 @@
 """
 ===============================================================================
 
-    Copyright (C) 2009-2018 Rudolf Cardinal (rudolf@pobox.com).
+    Original code copyright (C) 2009-2018 Rudolf Cardinal (rudolf@pobox.com).
 
     This file is part of cardinal_pythonlib.
 
@@ -22,11 +22,23 @@
 
 ===============================================================================
 
-WSGI middleware to disable client-side caching.
+**WSGI middleware to disable client-side caching.**
 
 """
 
 import logging
+
+from cardinal_pythonlib.wsgi.constants import (
+    TYPE_WSGI_APP,
+    TYPE_WSGI_APP_RESULT,
+    TYPE_WSGI_ENVIRON,
+    TYPE_WSGI_EXC_INFO,
+    TYPE_WSGI_RESPONSE_HEADERS,
+    TYPE_WSGI_START_RESPONSE,
+    TYPE_WSGI_START_RESP_RESULT,
+    TYPE_WSGI_STATUS,
+)
+
 log = logging.getLogger(__name__)
 log.addHandler(logging.NullHandler())
 # log.setLevel(logging.DEBUG)
@@ -38,21 +50,32 @@ log.addHandler(logging.NullHandler())
 # http://stackoverflow.com/questions/49547/making-sure-a-web-page-is-not-cached-across-all-browsers  # noqa
 # http://stackoverflow.com/questions/3859097/how-to-add-http-headers-in-wsgi-middleware  # noqa
 
-def add_never_cache_headers(headers):
+def add_never_cache_headers(headers: TYPE_WSGI_RESPONSE_HEADERS) -> None:
+    """
+    Adds WSGI headers to say "never cache this response".
+    """
     headers.append(("Cache-Control", "no-cache, no-store, must-revalidate"))  # HTTP 1.1  # noqa
     headers.append(("Pragma", "no-cache"))  # HTTP 1.0
     headers.append(("Expires", "0"))  # Proxies
 
 
 class DisableClientSideCachingMiddleware(object):
-    """WSGI middleware to disable client-side caching."""
+    """
+    WSGI middleware to disable client-side caching.
+    """
 
-    def __init__(self, app):
+    def __init__(self, app: TYPE_WSGI_APP) -> None:
         self.app = app
 
-    def __call__(self, environ, start_response):
+    def __call__(self,
+                 environ: TYPE_WSGI_ENVIRON,
+                 start_response: TYPE_WSGI_START_RESPONSE) \
+            -> TYPE_WSGI_APP_RESULT:
 
-        def custom_start_response(status, headers, exc_info=None):
+        def custom_start_response(status: TYPE_WSGI_STATUS,
+                                  headers: TYPE_WSGI_RESPONSE_HEADERS,
+                                  exc_info: TYPE_WSGI_EXC_INFO = None) \
+                -> TYPE_WSGI_START_RESP_RESULT:
             add_never_cache_headers(headers)
             log.debug("HTTP status {}, headers {}".format(status, headers))
             return start_response(status, headers, exc_info)
